/* Author: Matthias Walter (matthias.walter@listrik.de)
	  
Siehe http://eleccelerator.com/fusecalc/fusecalc.php?chip=atmega644p

Achtung: Im Ausliefrungszustand ist Clk/8 und 8MHZ intern aktiv.
D.h. ISP darf maximal 250KHz, besser sollten es 125KHz sein

Siehe auch: http://www.mikrocontroller.net/articles/AVR_Fuses
http://www.engbedded.com/fusecalc/
F:\Users\mwalter\Documents\Pedelec\Target\Fuse-Reset.odt
*/

#include <stdint.h>
#include <avr/interrupt.h>

#include "My_Includes.h"
#include "Main_MotorController2560_IO.h"

void PortsInit(void) {					//See "Main_MotorController_IO.h" for aliases
/*	A.0 = Input			AD0										Digital Pin 22 		Start/Reset
		A.1 = Output		AD1										Digital Pin	23		Buzzer
		A.2 = Input			AD2										Digital Pin	24		----
		A.3 = Input			AD3										Digital Pin	25		----
		A.4 = Input			AD4										Digital Pin	26		----
		A.5 = Output		AD5										Digital Pin	27		Bremsleuchte
		A.6 = Input			AD6										Digital Pin	28		----
		A.7 = Input			AD7										Digital Pin	29		----  					*/
	DDRA =  0b00100010;																												//Data direction. 0=Input, 1=Output
	PORTA = 0b11011100;																												//Turn internal Pull-Up ON=1 
	
/*	B.0 = Input			SS										Digital Pin	53		----
		B.1 = Input			SCK										Digital Pin 52		----
		B.2 = Input			MOSI									Digital Pin 51		----
		B.3 = Input			MISO									Digital Pin 50		----
		B.4 = Output		OC2A/PCINT4						Digital Pin 10		C+
		B.5 = Output		OC1A/PCINT5						Digital Pin 11		B+
		B.6 = Output		OC1B/PCINT6						Digital Pin 12		B-
		B.7 = Output		OC0A/OC1C/PCINT7			Digital Pin 13		A+/internal LED */
	DDRB =  0b11110000;																												//Data direction. 0=Input, 1=Output
  PORTB = 0b00000000;																												//Turn internal Pull-Up ON=1 
	
/*	C.0 = Input			A8										Digital Pin 37		----
		C.1 = Output		A9										Digital Pin 36		FAN
		C.2 = Input			A10										Digital Pin 35		Foot Switch
		C.3 = Input			A11										Digital Pin 34		----
		C.4 = Input			A12										Digital Pin 33		----
		C.5 = Input			A13										Digital Pin 32		----
		C.6 = Output		A14										Digital Pin 31		ExtLED2
		C.7 = Input			A15										Digital Pin 30		----						*/
	DDRC =  0b01000010;																												//Data direction. 0=Input, 1=Output
	PORTC = 0b10111000;																												//Turn internal Pull-Up ON=1 

/*	D.0 =	------		SCL										Digital Pin 21		---- 
		D.1 =	------		SDA										Digital Pin	20		---- 																
		D.2 =	Input			RX1										Digital Pin 19		----
		D.3 =	Output		TX1										Digital Pin 18		----
		D.4 = Input			CP1										nicht verfgbar
		D.5 = Input			XCK1									nicht verfgbar
		D.6 = Input			T1										nicht verfgbar
		D.7 = Input			T0										Digital Pin 38		----						*/
	DDRD =  0b00001000;																												//Data direction. 0=Input, 1=Output
	PORTD = 0b11110000;																												//Turn internal Pull-Up ON=1 
	
/*	E.0 =	Input			RX0										Digital Pin 0			----
		E.1 =	Output		TX0										Digital Pin 1			----
		E.2 =	Input			XCK0/AIN0							nicht verfgbar
		E.3 =	Input			OC3A/AIN1							Digital Pin 5			----		
		E.4 =	Input			OC3B/INT4 						Digital Pin 2			----
		E.5 = Input			OC3C/INT5							Digital Pin 3			----
		E.6 = Input			T3										nicht verfgbar
		E.7 = Input			CLK0									nicht verfgbar										*/
	DDRE =  0b00000010;																												//Data direction. 0=Input, 1=Output
	PORTE = 0b11111100;																												//Turn internal Pull-Up ON=1
	
	/*F.0 =	Input			ADC0									Analog Pin 0			I-Batt
		F.1 =	Input			ADC1									Analog Pin 1			U-Batt
		F.2 =	Input			ADC2									Analog Pin 2			FC-Temperature
		F.3 =	Input			ADC3									Analog Pin 3			Neigung
		F.4 = Input			ADC4/TCK							Analog Pin 4			JTAG
		F.5 = Input			ADC5/TMS							Analog Pin 5			JTAG
		F.6 = Input			ADC6/TDO							Analog Pin 6			JTAG
		F.7 = Input			ADC7/TDI							Analog Pin 7			JTAG						*/
	DDRF =  0b00000000;																												//Data direction. 0=Input, 1=Output
	PORTF = 0b00000000;																												//Turn internal Pull-Up ON=1
	
	/*G.0 =	Input					WR								Digital Pin 41		Display
		G.1 =	Input					RD								Digital Pin 40		Display
		G.2 =	Input					ALE								Digital Pin 39		Display
		G.3 =	Input					TOSC2							nicht verfgbar
		G.4 = Input					TOSC1							nicht verfgbar
		G.5 = Output				OC0B							Digital Pin 4			A-
		G.6 = nicht verfgbar									nicht verfgbar
		G.7 = nicht verfgbar									nicht verfgbar										*/
	DDRG =  0b00100000;																												//Data direction. 0=Input, 1=Output
	PORTG = 0b00011111;																												//Turn internal Pull-Up ON=1

	/*H.0 =	Input					RX2								Digital Pin 17		----
		H.1 =	Output				TX2								Digital Pin 16		----
		H.2 =	Input					XCK2							nicht verfgbar
		H.3 =	Input					OC4A							Digital Pin 6			----
		H.4 = Input					OC4B							Digital Pin 7			----
		H.5 = Output				OC4C							Digital Pin 8			Test DAC output 
		H.6 = Output				OC2B							Digital Pin 9			C-
		H.7 = Input					T4								nicht verfgbar								*/
	DDRH  = 0b01100010;																												//Data direction. 0=Input, 1=Output
	PORTH = 0b00011000;																												//Turn internal Pull-Up ON=1 
	
	/*J.0 =	Input					RX3/PCINT9				Digital Pin 15		Pedal Pulse	OR Foot switch																		
		J.1 =	Input					TX3/PCINT10				Digital Pin 14		Wheel Pulse 
		J.2 =	Input					XCK3/PCINT11			nicht verfgbar																			
		J.3 =	Input					PCINT12						nicht verfgbar																				
		J.4 = Input					PCINT13						nicht verfgbar																						
		J.5 = Input					PCINT14						nicht verfgbar																					
		J.6 = Input					PCINT15						nicht verfgbar																					
		J.7 = Input					xxxx							nicht verfgbar								*/
	DDRJ  = 0b00000000;																												//Data direction. 0=Input, 1=Output
	PORTJ = 0b00000000;																												//Turn internal Pull-Up ON=1
	
	/*K.0 =	Input					ADC08/PCINT16			Analog Pin 8			Motor-Temperature
		K.1 =	Input					ADC09/PCINT17			Analog Pin 9			----
		K.2 =	Input					ADC10/PCINT18			Analog Pin 10			----
		K.3 =	Input					ADC11/PCINT19			Analog Pin 11			----
		K.4 = Input					ADC12/PCINT20			Analog Pin 12			Hall-A
		K.5 = Input					ADC13/PCINT21			Analog Pin 13			Hall-B
		K.6 = Input					ADC14/PCINT22			Analog Pin 14			Hall-C
		K.7 = Input					ADC15/PCINT23			Analog Pin 15		  Hall-D			*/
	DDRK  = 0b00000000;																												//Data direction. 0=Input, 1=Output
	PORTK = 0b00000000;																												//Turn internal Pull-Up ON=1 
	
	/*L.0 =	Output				ICP4							Digital Pin 49		ExtLED1																									
		L.1 =	Input					ICPS							Digital Pin 48		----
		L.2 =	Input					T5								Digital Pin 47		----
		L.3 =	Input					OC5A							Digital Pin 46		----
		L.4 =	Input					OC5B							Digital Pin 45		----
		L.5 = Input					OC5C							Digital Pin 44		----
		L.6 =	Input					----							Digital Pin 43		----
		L.7 = Input					----							Digital Pin 42		----				*/
	DDRL  = 0b00000001;																												//Data direction. 0=Input, 1=Output
	PORTL = 0b11111110;																												//Turn internal Pull-Up ON=1 
}

void FlashInternalLED(uint8_t Cycles, uint16_t ON_Duration_ms, uint16_t OFF_Duration_ms) {
  uint8_t x; 
  /*	Cycles: Number of flashes
			Duration_ms: State duration in mS						*/
  for (x=0;x<Cycles;x++) {
		IntLED=1;				
		Waitms(ON_Duration_ms);
		IntLED=0;
		Waitms(OFF_Duration_ms);
  }
}

void FlashExternalLED(uint8_t FlashMode, uint8_t Cycles, uint16_t ON_Duration_ms, uint16_t OFF_Duration_ms) {
  uint8_t x; 
  /*
  Flash Modes:	0 = Flash ExtLED1 and ExtLED2 in parallel
								1 = Flash ExtLED1 and ExtLED2 alternately 
								2 = Flash ExtLED1 
 								3 = Flash ExtLED2 
  Cycles: Number of flashes
  Duration_ms: State duration in mS */	
	for (x=0; x<Cycles; x++) {
	  switch (FlashMode) {
			case 0:
				ExtLED1=1;
				ExtLED2=1;
				Waitms(ON_Duration_ms);
				ExtLED1=0;
				ExtLED2=0;
				Waitms(OFF_Duration_ms);
				break;
			
			case 1:
				ExtLED1=1;
				ExtLED2=0;
				Waitms(ON_Duration_ms);
				ExtLED1=0;
				ExtLED2=1;
				Waitms(OFF_Duration_ms);
				break;
			
			case 2:		
				ExtLED1=1;
				Waitms(ON_Duration_ms);
				ExtLED1=0;
				Waitms(OFF_Duration_ms);
				break;
			
			case 3:
				ExtLED2=1;
				Waitms(ON_Duration_ms);
				ExtLED2=0;
				Waitms(OFF_Duration_ms);
				break;
		}
  }
}
