//Author: Matthias Walter (matthias.walter@listrik.de)

#include <avr/io.h>

//Key debouncing
volatile uint8_t
	key_state,																																//debounced and (inverted) key state
	key_press,																																//key press detect. bitx = 1: keyx pressed
	key_rpt;																																	//key long press and repeat

//Key debounce
#define KEY_PIN         PINA
#define KEY0            0						//Bit 0
#define KEY1            1						//Bit 1
#define KEY2            2						//Bit 2
#define KEY3            3						//Bit 3
#define KEY4            4						//Bit 4
#define KEY5            5						//Bit 5
#define KEY6            6						//Bit 6
#define KEY7            7						//Bit 7
#define REPEAT_MASK     (1<<KEY1 | 1<<KEY2)       // Example for repeat: key1 and key2 must be pressed simultaneously
#define REPEAT_START    50                        // after 50*10mS = 500ms
#define REPEAT_NEXT     20                        // every 20*10mS = 200ms

struct mybits {
	uint8_t b0:1;
	uint8_t b1:1;
	uint8_t b2:1;
	uint8_t b3:1;
	uint8_t b4:1;
	uint8_t b5:1;
	uint8_t b6:1;
	uint8_t b7:1;
} __attribute__((__packed__));

#define BIT(r,n) (((volatile struct mybits *)&r)->b##n)
#define DDR(x) (*(&x - 1))																									// address of data direction register of port x
#define ToggleExternalLED1 ExtLED1=~ExtLED1;
#define ToggleExternalLED2 ExtLED2=~ExtLED2;
#define ToggleInternalLED IntLED=~IntLED;

void PortsInit(void);
void FlashExternalLED(uint8_t FlashMode, uint8_t Cycles, uint16_t ON_Duration_ms, uint16_t OFF_Duration_ms);
void FlashInternalLED(uint8_t Cycles, uint16_t ON_Duration_ms, uint16_t OFF_Duration_ms);

//************** IO Definitions **************
//KEY1 = Start_Reset = BIT(PINA,0)				//Digital Bit0, Pin 22
#define Buzzer				BIT(PORTA,1)				//Digital Pin 23
#define BrakeLight		BIT(PORTA,5)				//Digital Pin 27

#define A_Plus				BIT(PORTB,4)				//Digital Pin 10, OC2A
#define B_Minus		 		BIT(PORTB,5)				//Digital Pin 11, OC1A
#define B_Plus				BIT(PORTB,6)				//Digital Pin 12, OC1B
#define C_Minus				BIT(PORTB,7)				//Digital Pin 13, OC0A
#define IntLED				BIT(PORTB,7)				//Digital Pin 13 (can't use the LED because it is a PWM-output)
 
#define Fan						BIT(PORTC,1)				//Digital Pin 36
#define ExtLED2				BIT(PORTC,6)				//Digital Pin 31

#define C_Plus				BIT(PORTG,5)				//Digital Pin 4, OC0B

#define A_Minus				BIT(PORTH,6)				//Digital Pin 9, OC2B

#define Pedal_Pulse		BIT(PINJ,0)					//Communication Pin 15, PCINT9
#define Foot_Switch		Pedal_Pulse					//use the same pin
#define Wheel_Pulse		BIT(PINJ,1)					//Communication Pin 14, PCINT10

#define Hall_A				BIT(PINK,4)					//Analog Pin 12, PCINT20
#define Hall_B				BIT(PINK,5)					//Analog Pin 13, PCINT21
#define Hall_C				BIT(PINK,6)					//Analog Pin 14, PCINT22
#define Hall_D				BIT(PINK,7)					//Analog Pin 15, PCINT23

#define ExtLED1				BIT(PORTL,0)				//Digital Pin 49
